@@ -6,9 +6,11 @@
 
 #include <iterator>
 
+#include "wchar_t_short.h"
 #include <nsCOMPtr.h>
 #include <nsIDOMHTMLCollection.h>
 #include <nsIDOMNode.h>
+#include "wchar_t_default.h"
 
 class nsIDOMDocument;
 
